﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using CMS;
using CMS.Scheduler;
using CMS.EventLog;
using CMS.Helpers;
using CMS.DocumentEngine;
using CMS.DataEngine;
using CMS.Membership;
using System.Text;
using CMS.SiteProvider;
using CMS.MacroEngine;
using CMS.EmailEngine;
using System.Collections;

[assembly: RegisterCustomClass("Custom.OutdatedContentNotifierTask", typeof(Custom.OutdatedContentNotifierTask))]
namespace Custom
{
    public class OutdatedContentNotifierTask : ITask
    {
        /// <summary>
        /// Executes the task.
        /// </summary>
        /// <param name="ti">Info object representing the scheduled task</param>
        public string Execute(TaskInfo ti)
        {
            StringBuilder sb = new StringBuilder();
            StringBuilder sbOwner = new StringBuilder();
            List<OutdatedDocument> lstDocs = new List<OutdatedDocument>();
            try
            {
                // Get the task parameters
                string[] taskparams = ti.TaskData.Split(',');
                int intExpirationDays = ValidationHelper.GetInteger(taskparams[0], 0);
                bool blnSendNotification = ValidationHelper.GetBoolean(taskparams[1], false);
                bool blnDetail = ValidationHelper.GetBoolean(taskparams[2], false);

                // Get all the documents older than the specified number of days
                var outdateddocs = DocumentHelper.GetDocuments()
                    .Columns("DocumentName", "DocumentModifiedWhen", "NodeOwner", "NodeAliasPath")
                    .WhereLessThan("DocumentModifiedWhen", DateTime.Now.AddDays(-intExpirationDays))
                    .OnCurrentSite();
                
                // Loop through the documents
                foreach (var outdateddoc in outdateddocs)
                {
                    // Create a new object for each docuemnt, combining the document and owner data
                    OutdatedDocument doc = new OutdatedDocument();
                    doc.DocumentName = outdateddoc.DocumentName;
                    doc.DocumentModifiedWhen = outdateddoc.DocumentModifiedWhen;
                    doc.NodeAliasPath = outdateddoc.NodeAliasPath;
                    doc.OwnerID = outdateddoc.NodeOwner;

                    // Get the owner info
                    UserInfo uiOwner = UserInfoProvider.GetUserInfo(ValidationHelper.GetInteger(outdateddoc.NodeOwner, 0));
                    if (uiOwner != null)
                    {
                        doc.OwnerUsername = uiOwner.UserName;
                        doc.OwnerFullname = uiOwner.FullName;
                        doc.OwnerEmail= uiOwner.Email;
                    }
                    // Add the new object to the list
                    lstDocs.Add(doc);
                }

                // Set the total outdated content
                sb.Append("Total Outdated Content Items: " + outdateddocs.Count);

                // Get the list of unique owners from the collection 
                var owners = lstDocs.GroupBy(t => t.OwnerID)
                   .Select(grp => grp.First())
                   .ToList();
                
                // Loop through each owner to generate a list of ther documents
                foreach (var owner in owners)
                {
                    // Get all of docuemnts for the owner
                    var docs = lstDocs.FindAll(n => n.OwnerID == owner.OwnerID);

                    // Reorder list by oldest first
                    docs.Sort((x, y) => x.DocumentModifiedWhen.CompareTo(y.DocumentModifiedWhen));

                    if (blnDetail)
                    {
                        // Add each owner's username/count to the task results
                        sb.Append(" | ");
                        if (owner.OwnerUsername != null)
                        {
                            sb.Append(owner.OwnerUsername);
                        }
                        else
                        {
                            sb.AppendLine("UNASSIGNED");
                        }
                        sb.Append(" (" + docs.Count + ") | ");
                    }

                    // Build the list of outdated content for the owner
                    sbOwner.Clear();
                    foreach (var doc in docs)
                    {
                        sbOwner.Append(doc.DocumentModifiedWhen.ToString() + " - " + doc.DocumentName + "<br />" + doc.NodeAliasPath);
                        sbOwner.Append("<br /><br />");
                    }

                    // Determine if notifications should be sent
                    if ((blnSendNotification) && (owner.OwnerUsername != null))
                    {
                        // Send the report to the owner
                        EmailTemplateInfo eti = EmailTemplateProvider.GetEmailTemplate("OutdatedContentTemplate", SiteContext.CurrentSiteName);
                        // Add the custom variables to the MacroResolver
                        MacroResolver mcr = MacroResolver.GetInstance();
                        mcr.SetNamedSourceData("NAME", owner.OwnerFullname);
                        mcr.SetNamedSourceData("NUMBER_OF_DAYS", intExpirationDays);
                        mcr.SetNamedSourceData("OUTDATED_CONTENT_COUNT", docs.Count);
                        mcr.SetNamedSourceData("OUTDATED_CONTENT_LIST", sbOwner.ToString());

                        // Create message object
                        EmailMessage message = new EmailMessage();

                        // Get sender from settings
                        message.EmailFormat = EmailFormatEnum.Both;
                        message.From = eti.TemplateFrom;

                        // Do not send the e-mail if there is no sender specified
                        if (message.From != "")
                        {
                            // Initialize message
                            message.Recipients = owner.OwnerEmail;
                            message.Subject = eti.TemplateSubject;

                            // Send email via Email engine API
                            EmailSender.SendEmailWithTemplateText(SiteContext.CurrentSiteName, message, eti, mcr, true);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                sb.Append("ERROR: " + ex.Message);
            }

            return sb.ToString();
        }
    }

    public class OutdatedDocument
    {
        public string DocumentName { get; set; }
        public string NodeAliasPath { get; set; }
        public DateTime DocumentModifiedWhen { get; set; }
        public int OwnerID { get; set; }
        public string OwnerUsername { get; set; }
        public string OwnerEmail { get; set; }
        public string OwnerFullname { get; set; }
    }
}